# Microeconometrics
# CLASS 7
# Sample Selection
# Wiktor Budzinski

rm(list=ls())
# ctrl+l

# if the library is not installed on the given computer, delete the first '#'
# install.packages("psych")

# install.packages("sampleSelection")
# install.packages("stargazer)

# Loading libraries
library(psych)
library(sampleSelection)
library(stargazer)
library(censReg)
library(marginaleffects)
library(matlib)
library(mvtnorm)
library(matrixStats)
library(mfx)
library(GJRM)

setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2024\\Class 7")         # sets the access path (sometimes change the characters '\' to '/' is needed)
femlab <- readRDS("me.femlab.rds")

describe(femlab)

################################################# Exercise 1 ########################################
# Two part model + heckman

hist(femlab$wage)
summary(femlab$wage)
mean(femlab$wage == 0)

# basic regression
Eq1a <- wage ~ age + exper + expersq + educ + city
model0 <- lm(Eq1a,  data=femlab)
summary(model0)

# Estimate tobit
Eq1t <- wage ~ exper + expersq + educ + city + nwifeinc + mtr + unem + kidslt6 +kidsge6
model0b <- censReg(Eq1t, data=femlab) # Tobit
summary(model0b)

# Two part model
Eq1b <- inlf ~ age + exper + expersq + educ + city # selection model
model1 <- glm(Eq1b, data = femlab, family=binomial(link="probit"))
summary(model1)

Eq1c <- wage ~ age + exper + expersq + educ + city # selection model
model1b <- lm(Eq1c,  data=femlab[femlab$inlf == 1,])
summary(model1b)

# Heckman selection model (two-step estimation)
Eq1d<- inlf ~ age + exper + expersq + educ + city + nwifeinc + mtr + unem + kidslt6 +kidsge6  # selection model
model2 <- selection(Eq1d, Eq1a, data=femlab, method="2step")
summary(model2)

# MLE estimation
model2b <- selection(Eq1d, Eq1a, data=femlab, method="ml")
summary(model2b)

#  Models on logarithm of wages
hist(femlab$lwage)
summary(femlab$lwage)

# Two part model
Eq2 <- lwage ~ age + exper + expersq + educ + city
model3a <- lm(Eq2,  data=femlab[femlab$inlf == 1,])
summary(model3a)

# Heckman selection model (two-step estimation)
model3b <- selection(Eq1d, Eq2, data=femlab, method="2step")
summary(model3b)

## Marginal effects:
Eq1d<- inlf ~  exper + expersq + educ + city + nwifeinc + mtr + unem + kidslt6 +kidsge6  # selection model
model3c <- selection(Eq1d, Eq2, data=femlab, method="ml")
summary(model3c)
avg_slopes(model3c) # works only for ml
avg_slopes(model3c, type = 'link') # probit
avg_slopes(model3c, type = 'unconditional') # regression
# does not seem to give what we would expect

# Functions written by me
source("SampleSelMarg.R")
out <-  Eq2 # Not sure why I have to do that
sel<- Eq1d
amec<-AME(femlab, out, sel, 'log', "censored")
amec
amet<-AME(femlab, out, sel, 'log', "truncated")
amet

amec<-BootAme(100, femlab, out, sel, 'log', "censored")
amec
amet<-BootAme(100, femlab, out, sel, 'log', "truncated")
amet

# Comparison of ME with Tobit
me_tob<-margEff(model0b) 
summary(me_tob)

amec
################################################# Exercise 2 ##########################################
# What if education is endogenous
# Does not work with experience
Eq1d<- inlf ~ age  + educ + city + nwifeinc + mtr + unem + kidslt6 +kidsge6  # selection model
Eq2 <- lwage ~ age  + educ + city
model3b <- selection(Eq1d, Eq2, data=femlab, method="2step")
summary(model3b)

model4 <- heckit(Eq1d, Eq2, data = femlab, inst = ~ motheduc + fatheduc)
summary(model4) 

################################################# Exercise 3 ##########################################
# Sample selection in the case of binary outcome

rm(list=ls())
data("hiv", package = "GJRM")
hiv$wealth <- hiv$wealth/1000 # HIV data

# Religion as an exclusion restriction
sel<-hivconsent ~ highhiv + factor(religion) + age + education + wealth + marital + smoke + condom + std + knowsdiedofaids + region
m1 <- glm(sel, family=binomial(link="probit"), data = hiv)
summary(m1)

out<-hiv ~ highhiv + age + education + wealth + marital + smoke + condom + std + knowsdiedofaids  +region 
m2<- selection(sel, out, data = hiv)
summary(m2) # Why is there sigma (?)

hiv$hivx <- hiv$hiv == 1
out<-hivx ~ highhiv + age + education + wealth + marital + smoke + condom + std + knowsdiedofaids +region 
m2b<- selection(sel, out, data = hiv)
summary(m2b)