# Function calculates marginal effects from the sample selection model
  # Works with ML estimates
    # Uses bootstrap to obtain standard errors
    # AME produces Average Marginal Effects
    # BootAme produces AME + standard errors
  # Arguments:
    # data - data used in the model
    # out, sel are specifications for outcome and selection equations
    # form should be either "linear" or "log"
    # type should be either "truncated" or "censored" 

AME <- function(data, out, sel, form, type) {
  eps <- 0.00001
  environment(sel) <- environment()
  environment(out) <- environment()
  
  m <- selection(sel, out, data=data, method="ml")
  
  names <- unique(names(m$estimate))[2:11]
  Res <- matrix(, length(names), ncol = 1)
  P <-predict(m, newdata = data, # Its crucial to use newdata
             part = "outcome",
             type = "conditional")
  if(form == "log"){
    P <- exp(P + 0.5*m$estimate["sigma"]^2)
  }
  if(type == "truncated"){ # Truncated
    for(i in 1:length(names)){

      data_new <- data
      data_new[,names[i]] <- data_new[,names[i]] + eps
      P_new<-predict(m,newdata = data_new, 
                 part = "outcome",
                 type = "conditional")
      if(form == "log"){
        P_new <- exp(P_new + 0.5*m$estimate["sigma"]^2)
      }
      Res[i] <- mean((P_new[,2] - P[,2])/eps)
   
    }
  } else{ # Censored
    Psel<-predict(m, newdata = data,
               part = "selection",
               type = "response")
    for(i in 1:length(names)){
      data_new <- data
      data_new[,names[i]] <- data_new[,names[i]] + eps
      P_new<-predict(m,newdata = data_new, 
                     part = "outcome",
                     type = "conditional")
      if(form == "log"){
        P_new <- exp(P_new + 0.5*m$estimate["sigma"]^2)
      }
      Psel_new<-predict(m, newdata = data_new, 
                    part = "selection",
                    type = "response")
      
      Res[i] <- mean((Psel_new*P_new[,2] - Psel*P[,2])/eps, na.rm=TRUE)
      
    }
  }
  row.names(Res) <- names
  return(Res)
}

BootAme <- function(NSim, data, out, sel, form, type){
  
  AmeMain <- AME(data, out, sel, form, type)
  B_AME <- matrix(, nrow(AmeMain), ncol = NSim)
  for(i in 1:NSim){
    ID_i <- sample(1:nrow(data), nrow(data), replace = TRUE)
    data_i <- data[ID_i,]
    data_i <-data_i[order(data_i$inlf,decreasing=TRUE),] 
    B_AME[,i] <- AME(data_i, out, sel, form, type)
  }
  MAT = cbind(AmeMain, rowQuantiles(B_AME, probs = 0.05), rowQuantiles(B_AME, probs = 0.95))
  colnames(MAT)<-c("AME", "5%", "10%")
  return(MAT)
}